# File Maker for Photo Color Adjustment 16/8bit Version
# Ver. 2.92a
# (c)2020-21 Ohnishi, Yasuo
from ij import IJ 
from ij import plugin
from ij.io import DirectoryChooser
from ij.io import OpenDialog
from ij.gui import GenericDialog
import os

def convValue(Value, Mode):
#   Converting Luminance Value each in 8, 16, 32bit mode
#    print("value org: ", Value)
    if Mode == "16-bit":
#       print("16 Mode is","16-bit")
        Value = Value * 256
    elif Mode == "32-bit":
#       print("32 Mode is","32-bit")
        Value = Value * 256 * 256
    else:
#       print("Else Mode is", Mode)
        Value = Value
#   print("Converted value: ",Value, Mode)
    return Value
#
def convWinName(Name, filename, Mode):
#   Converting Windows Name each in 8, 16, 32bit mode
    if Mode == "8-bit":
        if Name == "C1-":
            Name = filename + " (red)"
        elif Name == "C2-":
            Name = filename + " (green)"
        elif Name == "C3-":
            Name = filename + " (blue)"
    else:
        Name = Name + filename
#   print ("Windows Names:",Name)
    return Name

#####################################################
def RGBmanipulation(folder,filename,parm):
#  input Parameters
    if parm is not None:
 # if parm is not null object, then...
        Mode, parm_Binary, parm_Distance, parm_Dark = parm
        print(Mode, parm_Binary, parm_Distance, parm_Dark) 
#     Mode: Image Format 16-bit or 8-bit
#     parm_Binary: Threshold of fore / background
#     parm_Distance: Lightness of Distance adjusting Layer
    if Mode == "8-bit":
        IJ.open(folder + filename)
    else:
        IJ.run("Bio-Formats Importer", "open=\'"+ folder + filename + "\' autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT")
    IJ.run("Split Channels")
    IJ.run(Mode)
# R ch.
    IJ.selectWindow(convWinName("C1-", filename, Mode)) 
    imp0 =  IJ.getImage()
    IJ.saveAsTiff(imp0, folder + filename +"_R.tif")
# G ch.
    IJ.selectWindow(convWinName("C2-", filename, Mode)) 
    imp1 =  IJ.getImage()
    IJ.saveAsTiff(imp1, folder + filename +"_G.tif")
# B ch. & B x 0.3
    IJ.selectWindow(convWinName("C3-", filename, Mode)) 
    imp2 =  IJ.getImage()
    IJ.saveAsTiff(imp2, folder + filename +"_B.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
 #        Open "Clipboard" Window as B x 0.3 Image
    IJ.run("Multiply...", "value=0.3")
    impB03 = IJ.getImage()
# G ch. x 0.7
    IJ.selectWindow(filename +"_G.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
 #        Open "Clipboard-1" Window as G x 0.7 Image
    IJ.run("Multiply...", "value=0.7")
    impG07 = IJ.getImage()
# B x 0.3 + G x 0.7
    ic = plugin.ImageCalculator()
    imp3 = ic.run("Add create", impB03, impG07) # No window made
    IJ.saveAsTiff(imp3, folder + filename +"_B_foreground.tif")
    imp3.close()
# Close G x 0.7 image windows
    IJ.selectWindow("Clipboard-1")
    IJ.run("Close")
    impG07.close()
# (G ch.+30) x 0.7
    IJ.selectWindow(filename +"_G.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
 #        Open "Clipboard-1" Window as (G+30) x 0.7 Image
    IJ.run("Add...", "value=" + str(convValue(30, Mode))) 
    IJ.run("Multiply...", "value=0.7")
    impG30x07 = IJ.getImage()
# B x 0.3 + (G+30) x 0.7
    ic = plugin.ImageCalculator()
    imp4 = ic.run("Add create", impB03, impG30x07) # No window made
    IJ.saveAsTiff(imp4, folder + filename +"_Bx0.3+(G+30)x0.7.tif")
    imp4.close()
#
# add B x 0.3 + (G+30) x 0.7 + 20
    IJ.open(folder + filename +"_Bx0.3+(G+30)x0.7.tif")
    IJ.run("Add...", "value=" + str(convValue(parm_Distance, Mode))) # add Value parm_Distance
    imp4a = IJ.getImage()
    IJ.saveAsTiff(imp4a, folder + filename +"_B_Background.tif")
    IJ.selectWindow(filename +"_B_Background.tif")
    IJ.run("Close")
    imp4a.close()
#########################################
# Close (G+30) x 0.7 image windows
    IJ.selectWindow("Clipboard-1")
    IJ.run("Close")
    impG30x07.close()
# Close B x 0.3 image windows
    IJ.selectWindow("Clipboard")
    IJ.run("Close")
    impB03.close()
# G ch. x 0.5
    IJ.selectWindow(filename +"_G.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
    IJ.run("Multiply...", "value=0.5")
 #        Open "Clipboard" Window as G x 0.5 Image
    impG05 = IJ.getImage()
# R ch. x 0.5
    IJ.selectWindow(filename +"_R.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
    IJ.run("Multiply...", "value=0.5")
 #        Open "Clipboard-1" Window as R x 0.5 Image
    impR05 = IJ.getImage()
# B x 0.5 + R x 0.5
    ic = plugin.ImageCalculator()
    imp5 = ic.run("Add create", impR05, impG05) # No window made
    IJ.saveAsTiff(imp5, folder + filename +"_G+R.tif")
# Periphery Adjustment Mask: B - ((R+G)/2-10)
    IJ.open(folder + filename +"_G+R.tif")
    IJ.selectWindow(filename +"_G+R.tif")
    IJ.run("Subtract...", "value="+ str(convValue(10, Mode))) # subtract Value 10
    impGR = IJ.getImage()
    ic = plugin.ImageCalculator()
    imp2GR = ic.run("Subtract create", imp2, impGR) #imp2:B - impGR:(G+R/2)
    imp2GR.show()  # Show the subtracted image in window
    IJ.run("Multiply...", "value=8") # Save the Subtracted image x 8
    IJ.saveAsTiff(imp2GR, folder + filename +"_Periphery_Adjst_Mask.tif") 
#   Close Periphery Adjustment Mask Window
    IJ.selectWindow(filename +"_Periphery_Adjst_Mask.tif") 
    IJ.run("Close")
    imp2GR.close()
##############
# Close G x 0.5 & R x 0.5 windows
    IJ.selectWindow("Clipboard")
    IJ.run("Close")
    IJ.selectWindow("Clipboard-1")
    IJ.run("Close")
    impG05.close()
    impR05.close()
# Dark Parts 0 - parm_Dark
    IJ.open(folder + filename +"_G+R.tif")
    IJ.run("Add...", "value=" + str(convValue(255 - parm_Dark, Mode))) 
    IJ.run("Subtract...", "value=" + str(convValue(255 - parm_Dark, Mode)))
    imp6 = IJ.getImage()
    IJ.saveAsTiff(imp6, folder + filename +"_Dark.tif")
# Close Dark Parts window
    IJ.selectWindow(filename +"_Dark.tif")
    IJ.run("Close")
    imp6.close()
# R + 128
    IJ.open(folder + filename +"_R.tif")
    IJ.run("Add...", "value=" + str(convValue(128, Mode)))
    imp8 = IJ.getImage()
    IJ.saveAsTiff(imp8, folder + filename +"_R+128.tif")
# Make Binary Mask: Threshold R parm_Binary
    IJ.open(folder + filename +"_R.tif")
    IJ.run("Add...", "value=" + str(convValue(255 - parm_Binary, Mode)))
    IJ.setThreshold(convValue(256, Mode)-1, convValue(256, Mode)-1, "Black & White")
    IJ.run("Convert to Mask")
    IJ.run("Invert")
    IJ.saveAs("Tiff", folder + filename +"_Binary_Mask.tif")
# Close Binary Mask R128 windows
    IJ.selectWindow(filename +"_Binary_Mask.tif")
    IJ.run("Close")
    IJ.selectWindow(filename +"_R+128.tif")
    IJ.run("Close")
    imp8.close()
    IJ.selectWindow(filename +"_G+R.tif")
    IJ.run("Close")
    impGR.close()
    IJ.selectWindow(filename +"_R.tif")
    IJ.run("Close")
    imp0.close()
    IJ.selectWindow(filename +"_G.tif")
    IJ.run("Close")
    imp1.close()
    IJ.selectWindow(filename +"_B.tif")
    IJ.run("Close")
    imp2.close()
    os.remove(folder + filename +"_Bx0.3+(G+30)x0.7.tif")
   
#####################################################

# Get parameters from the input DialogBox 
def getParameters():
	gd = GenericDialog("Enter Parameters")
	mode = ["16-bit","8-bit"]
	gd.addChoice("Image Format (16bit or 8bit [x 3ch.])", mode, mode[0])
	gd.addNumericField("Background Threshold (0 to 255)", 128, 0)
	gd.addNumericField("Background Lightness (Max to 100)", 10, 0)
	gd.addNumericField("Dark part Threshold (0 to 255)", 50, 0)
	gd.showDialog()

	if gd.wasCanceled():  
		parm_Mode   = "Stop"
		parm_Binary   = 128
		parm_Distance = 10
		parm_Dark = 50
		IJ.log("Canceled")
		return str(parm_Mode), parm_Binary, parm_Distance, parm_Dark
		
	parm_Mode   = gd.getNextChoice()
	parm_Binary   = gd.getNextNumber()
	parm_Distance = gd.getNextNumber()
	parm_Dark = gd.getNextNumber()
	if parm_Binary > 255: #Set Max Value
	    parm_Binary = 255
	if parm_Distance > 100:
	    parm_Binary = 100
	if parm_Dark > 255: #Set Max Value
	    parm_Dark = 255
	return str(parm_Mode), parm_Binary, parm_Distance, parm_Dark

# Main
od = OpenDialog("Choose a file")
folder = od.getDirectory()
file = od.getFileName()
path = folder + file
IJ.log(path)
IJ.log(folder)
IJ.log(file)
parm = getParameters()
print ("Parm[0]=", parm[0])
if parm[0] <> "Stop":
	RGBmanipulation(folder,file,parm)
	IJ.log("Finish")
	IJ.run("Quit")
